@@ -1,5 +1,4 @@
 
-
 import crocodile.toolbox as tb
 import platform
 import os
@@ -28,7 +27,8 @@ ProgramFilesX86 = P(tmp) if (tmp := os.getenv("ProgramFiles(x86)")) else None  #
 
 CommonProgramFiles = P(tmp) if (tmp := os.getenv("CommonProgramFiles")) else None  # C:\Program Files\Common Files
 CommonProgramW6432 = P(tmp) if (tmp := os.getenv("CommonProgramW6432")) else None  # C:\Program Files\Common Files
-CommonProgramFilesX86 = P(tmp) if (tmp := os.getenv("CommonProgramFiles(x86)")) else None  # C:\Program Files (x86)\Common Files
+CommonProgramFilesX86 = P(tmp) if \
+    (tmp := os.getenv("CommonProgramFiles(x86)")) else None  # C:\Program Files (x86)\Common Files
 
 Tmp = P(tmp) if (tmp := os.getenv("TMP")) else None  # C:\Users\usernrame\AppData\Local\Temp
 Temp = Tmp
@@ -45,7 +45,6 @@ UserProfile = P(tmp) if (tmp := os.getenv("USERPROFILE")) else None  # e.g C:\Us
 HomePath = P(tmp) if (tmp := os.getenv("HOMEPATH")) else None  # e.g. C:\Users\eng_a
 Public = P(tmp) if (tmp := os.getenv("PUBLIC")) else None  # C:\Users\Public
 
-
 OneDriveConsumer = P(tmp) if (tmp := os.getenv("OneDriveConsumer")) else None
 OneDriveCommercial = P(tmp) if (tmp := os.getenv("OneDriveCommercial")) else None
 OneDrive = P(tmp) if (tmp := os.getenv("OneDrive")) else None
@@ -57,12 +56,13 @@ OneDriveExe = LocalAppData.joinpath("Microsoft/OneDrive/OneDrive.exe") if LocalA
 
 def get_address():
     netifaces = tb.install_n_import("netifaces")
-    subnet_mask = netifaces.ifaddresses(netifaces.gateways()['default'][netifaces.AF_INET][1])[netifaces.AF_INET][0]['netmask']
+    subnet_mask = netifaces.ifaddresses(netifaces.gateways()['default'][netifaces.AF_INET][1])[netifaces.AF_INET][0][
+        'netmask']
     default_gateway = netifaces.gateways()['default'][netifaces.AF_INET][0]
 
     import uuid
     mac = uuid.getnode()
-    mac_address = ":".join(("%012X" % mac)[i:i+2] for i in range(0, 12, 2))
+    mac_address = ":".join(("%012X" % mac)[i:i + 2] for i in range(0, 12, 2))
     # elif hex_format: return hex(mac)
     # else: return mac
 
@@ -101,8 +101,7 @@ class EnvVar:
     def set(key, val, temp=False, run=False):
         if system == "Windows":
             if temp is False:
-                res = f"setx {key} {val}"  # WARNING: setx limits val to 1024 characters
-                # in case the variable included ";" separated paths, this limit can be exceeded.
+                res = f"setx {key} {val}"  # WARNING: setx limits val to 1024 characters # in case the variable included ";" separated paths, this limit can be exceeded.
                 return res if not run else tm.run(res, shell="powershell")
             else:
                 raise NotImplementedError
@@ -115,6 +114,7 @@ class EnvVar:
     def get(key, run=False):
         result = f"${key}"  # works in powershell and bash
         return result if run is False else tm.run(result, shell="powershell")
+
     # in windows cmd `%key%`
 
     @staticmethod
@@ -142,9 +142,7 @@ class PathVar:
             elif kind == "replace":  # Replace the Path variable in the current window (use with caution!):
                 command = fr'$env:Path = "{path}"'
             else: raise KeyError
-            result = command
-            return result if run is False else tm.run(result, shell="powershell")
-
+            return command if run is False else tm.run(command, shell="powershell")
         else: result = f'export PATH="{path}:$PATH"'
         return result if run is False else tm.run(result, shell="powershell")
 
@@ -156,9 +154,7 @@ class PathVar:
             command = fr'[Environment]::SetEnvironmentVariable("Path", $env:PATH + ";{path}", "{scope}")'
             result = backup + command
             return result if run is False else tm.run(result, shell="powershell")
-
-        else:
-            tb.P.home().joinpath(".bashrc").append_text(f"export PATH='{path}:$PATH'")
+        else: tb.P.home().joinpath(".bashrc").append_text(f"export PATH='{path}:$PATH'")
 
     @staticmethod
     def set_permanetly(path, scope=["User", "system"][0], run=False):
@@ -191,36 +187,16 @@ class PathVar:
 def get_shell_profiles(shell):
     # following this: https://docs.microsoft.com/en-us/powershell/module/microsoft.powershell.core/about/about_profiles?view=powershell-7.2
     # https://devblogs.microsoft.com/scripting/understanding-the-six-powershell-profiles/
-
     # Dynmaically obtained:
-    profiles = tb.Struct(
-        CurrentUserCurrentHost = tm.run("$PROFILE.CurrentUserCurrentHost", shell=shell).as_path,
-        CurrentUserAllHosts = tm.run("$PROFILE.CurrentUserAllHosts", shell=shell).as_path,
-        AllUsersCurrentHost = tm.run("$PROFILE.AllUsersCurrentHost", shell=shell).as_path,
-        AllUsersAllHosts = tm.run("$PROFILE.AllUsersAllHosts", shell=shell).as_path,
+    return tb.Struct(
+        CurrentUserCurrentHost=tm.run("$PROFILE.CurrentUserCurrentHost", shell=shell).as_path,
+        CurrentUserAllHosts=tm.run("$PROFILE.CurrentUserAllHosts", shell=shell).as_path,
+        AllUsersCurrentHost=tm.run("$PROFILE.AllUsersCurrentHost", shell=shell).as_path,
+        AllUsersAllHosts=tm.run("$PROFILE.AllUsersAllHosts", shell=shell).as_path,
     )
 
-    # Static:
-    # profiles = dict(
-    #     Windows = tb.Struct(CurrentUserCurrentHost=tb.P(r"$PSHOME\Profile.ps1"),
-    #                           CurrentUserAllHosts=tb.P(r"$Home\[My ]Documents\PowerShell\Profile.ps1"),
-    #                           AllUsersCurrentHost=tb.P(r"$PSHOME\Microsoft.PowerShell_profile.ps1"),
-    #                           AllUsersAllHosts=tb.P(r"$Home\[My ]Documents\PowerShell\Microsoft.PowerShell_profile.ps1")),
-    # Linux = tb.Struct(CurrentUserCurrentHost=tb.P(r""),
-    #                     CurrentUserAllHosts=tb.P(r""),
-    #                     AllUsersCurrentHost=tb.P(r""),
-    #                     AllUsersAllHosts=tb.P(r"")),
-    # macOS = tb.Struct(CurrentUserCurrentHost=tb.P(r""),
-    #                     CurrentUserAllHosts=tb.P(r""),
-    #                     AllUsersCurrentHost=tb.P(r""),
-    #                     AllUsersAllHosts=tb.P(r"")),
-    # )[system]
-    return profiles
 
-
-def construct_path(path_list):
-    from functools import reduce
-    return reduce(lambda x, y: x + sep + y, tb.L(tb.pd.unique(path_list)).apply(str))
+def construct_path(path_list): return tb.L(__import__("pd").unique(path_list)).reduce(lambda x, y: str(x) + sep + str(y))
 
 
 if __name__ == '__main__':
